
'use strict';

var path = require('path');
var Router = require('koa-router');
var resolve=path.resolve;
var fs = require('mz/fs');

let router = new Router();

const mkdir = (list, index, callback = () => {}) => {
  if (index >= list.length) {
    callback();
    return false;
  }
  index += 1;
  const path = list.slice(0, index).join('/');
  fs.stat(path || '/', (err, stat) => {
    if (err) {
      fs.mkdir(path, () => {
        mkdir(list, index, callback);
      });
      return false;
    }
    if (stat.isDirectory()) {
      mkdir(list, index, callback);
    } else if (stat.isFile()) {
      callback();
      return false;
    }
  });
};
exports.mkdir = mkdir;

const setErrorLog = (logPath,data) => {
  if(!logPath){
    return false;
  }
  const list = logPath.split('/');
  mkdir(list.slice(0, list.length - 1), 0, () => {
    fs.appendFile(logPath, `\r\n${JSON.stringify(data)}`, (err) => {
      if (err) {
        console.log('写文件操作失败');
      } else {
        console.log('写文件操作成功');
      }
    });
  });
};

const initSuccess = (callback,logPath, defaultCode) => async (ctx) => {
  try {
    await callback(ctx);
  } catch (e) {
    console.log(e,defaultCode);
    setErrorLog(logPath,{
      url: ctx.path,
      body: ctx.request.body,
      method: ctx.method,
      message: e.sqlMessage || e.message,
    });
    ctx.body = e.body || {
      code: defaultCode,
      data: {},
      message: e.sqlMessage || e.message,
    };
  }
};

async function bind(url,opts){
  let stats=await fs.stat(url);
  if(stats.isFile()){
    let controller=require(url);
    controller.forEach(res=>{
      router[res.type || opts.type](res.url,initSuccess(res.success, opts.logPath, opts.defaultCode));
    })
  }else if(stats.isDirectory()){
    let dir=await fs.readdir(url);
    dir.forEach(res=>{
      bind(resolve(url,res),opts)
    });
  }
}

/**
 *
 * @param {String} root
 * @param {Object} [app]
 * @param {Object} [opts]
 * @return
 * @api public
 */

module.exports =async (root='', app, opts)=> {
  app.use(router.routes()).use(router.allowedMethods());
  try{
    bind(root, {...{
      type:'get',
      logPath: '',
      defaultCode: '-1'
    }, ...opts})
  }catch(e){
    console.log(e);
  }
}
